"""Web interface for Peloterm."""

# Remove automatic imports to prevent RuntimeWarning when running as module
# from .server import start_server, broadcast_metrics, stop_server

# __all__ = ['start_server', 'broadcast_metrics', 'stop_server'] 